import config from '@/application/config'
import type { HttpResponse, TextureData, UUID } from '@/application/types'
import { unduplicateArray } from '@/application/utilities'
import { loadTexture } from '@/composables/gameComposable'
import { MapStorage } from '@/storage/storages'

import Tilemap = Phaser.Tilemaps.Tilemap
import TilemapLayer = Phaser.Tilemaps.TilemapLayer
import Tileset = Phaser.Tilemaps.Tileset
import Tile = Phaser.Tilemaps.Tile

export function getTile(layer: TilemapLayer | Tilemap, x: number, y: number): Tile | undefined {
  const tile = layer.getTileAtWorldXY(x, y)
  if (!tile) return undefined
  return tile
}

export function tileToWorldXY(layer: TilemapLayer | Tilemap, pos_x: number, pos_y: number) {
  const worldPoint = layer.tileToWorldXY(pos_x, pos_y)
  if (!worldPoint) return { positionX: 0, positionY: 0 }

  const positionX = worldPoint.x + config.tile_size.y
  const positionY = worldPoint.y

  return { positionX, positionY }
}

export function tileToWorldX(layer: TilemapLayer | Tilemap, pos_x: number, pos_y: number): number {
  const worldPoint = layer.tileToWorldXY(pos_x, pos_y)
  if (!worldPoint) return 0

  return worldPoint.x + config.tile_size.x / 2
}

export function tileToWorldY(layer: TilemapLayer | Tilemap, pos_x: number, pos_y: number): number {
  const worldPoint = layer.tileToWorldXY(pos_x, pos_y)
  if (!worldPoint) return 0

  return worldPoint.y + config.tile_size.y * 1.5
}

/**
 * Can also be used to replace tiles
 * @param map
 * @param layer
 * @param x
 * @param y
 * @param tileName
 */
export function placeTile(map: Tilemap, layer: TilemapLayer, x: number, y: number, tileName: string) {
  let tileImg = map.getTileset(tileName) as Tileset
  if (!tileImg) {
    tileImg = map.getTileset('blank_tile') as Tileset
  }
  layer.putTileAt(tileImg.firstgid, x, y)
}

export function setLayerTiles(map: Tilemap, layer: TilemapLayer, tiles: string[][]) {
  if (!tiles) return

  tiles.forEach((row: string[], y: number) => {
    row.forEach((tile: string, x: number) => {
      placeTile(map, layer, x, y, tile)
    })
  })
}

export function createTileArray(width: number, height: number, tile: string = 'blank_tile') {
  return Array.from({ length: height }, () => Array.from({ length: width }, () => tile))
}

export const calculateIsometricDepth = (x: number, y: number, width: number = 0, height: number = 0, isCharacter: boolean = false) => {
  const baseDepth = x + y
  if (isCharacter) {
    return baseDepth // @TODO: Fix collision, this is a hack
  }
  return baseDepth + (width + height) / (2 * config.tile_size.x)
}

export function FlattenMapArray(tiles: string[][]) {
  const normalArray = []

  for (const row of tiles) {
    normalArray.push(...row)
  }

  return normalArray
}

export async function loadMapTilesIntoScene(map_id: UUID, scene: Phaser.Scene) {
  const mapStorage = new MapStorage()
  const map = await mapStorage.get(map_id)
  if (!map) return

  const tileArray = unduplicateArray(FlattenMapArray(map.tiles))

  // Load each tile into the scene
  for (const tile of tileArray) {
    const textureData = {
      key: tile,
      data: '/textures/tiles/' + tile + '.png',
      group: 'tiles',
      updatedAt: map.updatedAt // @TODO: Fix this
    } as TextureData
    await loadTexture(scene, textureData)
  }
}
